/* This file is part of the Views Template Library, written by
 * Gary Powell & Martin Weiser
 *
 * Copyright (c) 1999  Gary Powell
 * Copyright (c) 1999  Konrad Zuse Zentrum fuer Informationstechnik Berlin
 *
 * This material is provided "as is", with absolutely no warranty expressed
 * or implied. Any use is at your own risk.
 *
 * Permission to use or copy this software for any purpose is hereby granted 
 * without fee, provided the above notices are retained on all copies.
 * Permission to modify the code and to distribute modified code is granted,
 * provided the above notices are retained, and a notice that the code was
 * modified is included with the above copyright notice.
 */

//
// Traits classes that map container types and constness tags to iterator types.
//

#if!defined(_VIEW_BASE_DETAIL_HPP)
#define _VIEW_BASE_DETAIL_HPP

#if !defined(_VIEW_BASE_HPP)
#include "view_base.hpp"
#endif

#if !defined(_VIEW_BASE_TRAITS_HPP)
#include "details/view_base_traits.hpp"
#endif

#if !defined(_VIEW_TRAITS_HPP)
#include "details/view_traits.hpp"
#endif

#if !defined(_STL_ALGORITHM_FWD_HPP)
#include "details/stl_algorithm_fwd.hpp"
#endif

BEGIN_VIEW_NAMESPACE
//
// Tag classes used for telling views whether their referenced containers
// should be const or mutable.
//
class mutable_view_tag {};
class const_view_tag {};


//
// This is a traits class for determining whether to use the type, or const_type
// of a trait.
//
#if defined(HAVE_TEMPLATE_TEMPLATE)
template < class raw_type,
           template<class T> class traits_template,
           class const_tag >
struct select_type { typedef typename traits_template<raw_type>::const_type type; };

template < class raw_type,
           template<class T> class traits_template>
struct select_type< raw_type, traits_template, mutable_view_tag > 
{ typedef typename traits_template<raw_type>::type type; };
#else
template < class raw_type,
           class traits_template,
           class const_tag >
struct select_type 
{ 
	typedef typename traits_template::const_type type; 
};

template < class raw_type,
           class traits_template>
struct select_type< raw_type, traits_template, mutable_view_tag > 
{ 
	typedef typename traits_template::type type; 
};
#endif

//
// A simple utility class that allows operator->() to be defined for
// iterators that return temporaries.
//
template <class T>
class member_access_proxy {
  T t;
public:
  typedef reference_traits<T>::const_type const_reference;
  typedef pointer_traits<T>::type         pointer;
  typedef pointer_traits<T>::const_type   const_pointer;
  
  member_access_proxy(const_reference t_)
    : t(t_) {}
  pointer       operator->()       { return &t; }
  const_pointer operator->() const { return &t; }
};


#if !defined(USE_NON_STANDARD_SWAP)

#if defined (USE_VIEW_NAMESPACE)
// otherwise this will collide with ::swap.
template<class T>
void swap(T&a,T&b)
{
	std::swap(a,b);
}
#endif // USE_VIEW_NAMESPACE
#endif // !USE_NON_STANDARD_SWAP

END_VIEW_NAMESPACE

#endif // _VIEW_BASE_H
// $Id: view_base_detail.hpp,v 1.1 2000/11/20 14:11:13 bzfweise Exp $
