/* -*- c++ -*-
 * Copyright (c) 1999  
 * Gary Powell, Martin Weiser
 *
 * This material is provided "as is", with absolutely no warranty expressed
 * or implied. Any use is at your own risk.
 *
 * Permission to use or copy this software for any purpose is hereby granted 
 * without fee, provided the above notices are retained on all copies.
 * Permission to modify the code and to distribute modified code is granted,
 * provided the above notices are retained, and a notice that the code was
 * modified is included with the above copyright notice.
 *
 */
#if !defined(_DOWNCAST_VIEW_HPP)
#define _DOWNCAST_VIEW_HPP	1

#if !defined(_DOWNCAST_VIEW_FWD_HPP)
#include "downcast_view_fwd.hpp"	// validate that the forward declaration matches the actual one.
#endif

#if !defined(_VIEW_TRAITS_HPP)
#include "details/view_traits.hpp"
#endif

#if !defined(_FILTER_VIEW_HPP)
#include "filter_view.hpp"
#endif

#if !defined(_FUNCTORS_HPP)
#include "functors.hpp"
#endif

#if !defined(_POLYMORPHIC_VIEW_HPP)
#include "polymorphic_view.hpp"
#endif

BEGIN_VIEW_NAMESPACE
BEGIN_VIEW_DETAILS_NAMESPACE
//
// This struct is misused as a local namespace to ease the writing of all
// those typedefs.
//
template <class container,
          class derivedType,
          class const_tag,
          class iterator_tag,
          class ownership_tag>
class downcast_namespace {
private:  
  typedef typename view_traits<container,const_tag>::container_type domain_type;

  // We need the type the iterators in the base containers point to. Since
  // std::iterator_traits<T const*>:: value_type seems to be T instead of T const,
  // we have to do some trickery to know whether or not the return type of
  // the iterator dereferencing is actually const or not. (I don't understand
  // the rationale behind the standard here, it seems plain broken. Or is it
  // just the SGI implementation as of gcc 2.95? mw)
  typedef depointer_traits<std::iterator_traits<typename domain_type::value_type>::pointer>::type baseType;
  typedef downcast<baseType,derivedType> downcast_fn;

  typedef downcast_fn::result_type derivedPointerType;
  typedef transform_view<domain_type, downcast_fn,
                         const_tag, ownership_tag, derivedPointerType> downcasted;
  
  typedef is_nonzero<derivedPointerType> is_nonzero_fn;
  typedef filter_view<downcasted, is_nonzero_fn, const_tag, iterator_tag, aggregated_ownership> filtered;
  
  typedef polymorphic_view<filtered, const_tag, aggregated_ownership> dereferenced;

protected:
  //
  // Here comes the view itself. Its main purpose compared to "dereferenced"
  // is the convenient constructor.
  //
  class view: public dereferenced {
  public:   
    typedef ctor_arg<container,const_tag,ownership_tag>::type ctor_arg_type;

    explicit view() {}
    explicit view(ctor_arg_type& cont)
      : dereferenced(filtered(downcasted(cont,
					 downcast_fn()),
			      is_nonzero_fn())) {}
    
  };
};

END_VIEW_DETAILS_NAMESPACE

template <class container,
          class derivedType,
          class const_tag,
          class iterator_tag,
          class ownership_tag>
class downcast_view
	: public VIEW_DETAILS_NAMESPACE::downcast_namespace<container,derivedType,const_tag,iterator_tag,ownership_tag>::view {
  typedef VIEW_DETAILS_NAMESPACE::downcast_namespace<container,derivedType,const_tag,iterator_tag,ownership_tag>::view inherited;
public:
  typedef ctor_arg<container,const_tag,ownership_tag>::type ctor_arg_type;

  // containers have to be default constructible
  explicit downcast_view() {}
  explicit downcast_view(ctor_arg_type& cont)
    : inherited(cont) {}
};

END_VIEW_NAMESPACE
#endif // _DOWNCAST_VIEW_H
