/* This file is part of the Views Template Library, written by
 * Gary Powell & Martin Weiser
 *
 * Copyright (c) 2000  Gary Powell
 * Copyright (c) 2000  Konrad Zuse Zentrum fuer Informationstechnik Berlin
 *
 * This material is provided "as is", with absolutely no warranty expressed
 * or implied. Any use is at your own risk.
 *
 * Permission to use or copy this software for any purpose is hereby granted
 * without fee, provided the above notices are retained on all copies.
 * Permission to modify the code and to distribute modified code is granted,
 * provided the above notices are retained, and a notice that the code was
 * modified is included with the above copyright notice.
 */

#if !defined(_FLAGGED_FILTER_VIEW_HPP)
#define _FLAGGED_FILTER_VIEW_HPP 1

#if !defined(_FLAGGED_FILTER_VIEW_FWD_HPP)
#include "flagged_filter_view_fwd.hpp"	// validate that the forward declaration matches the actual one.
#endif

#if !defined(_FILTER_VIEW_HPP)
#include "filter_view.hpp"
#endif

#if !defined(_ZIP_VIEW_HPP)
#include "zip_view.hpp"
#endif

#if !defined(_TRANSFORM_VIEW_HPP)
#include "transform_view.hpp"
#endif

#if !defined(STL_SGIEXT_HPP)
#include "stl_sgiext.hpp"
#endif

BEGIN_VIEW_NAMESPACE
BEGIN_VIEW_DETAILS_NAMESPACE

//
// A view that presents only those elements that are flagged by an
// additional controlling container.
//

template <class container,
          class flags,
          class const_tag,
          class iterator_tag,
          class ownership_tag_a,
          class ownership_tag_b
>
class flagged_filter_namespace {
           
  // Define a zip view of the two containers to create (data,flag) pairs.
  typedef zip_view<container,flags,const_tag,reference_pair,iterator_tag,
                   ownership_tag_a,ownership_tag_b>  pair_view;

  typedef select2nd<typename pair_view::value_type> select_flag;


  // Define a filtered view of the (data,flag) pairs presenting only the
  // (data,true) pairs.
  typedef filter_view<pair_view,select_flag,
                      const_tag,iterator_tag,aggregated_ownership>  filtered_pairs;
  
  typedef select1st<typename filtered_pairs::value_type> select_data;
  

  // Define a transform view presenting the data part of the (data,true) pairs.
  typedef transform_view<filtered_pairs,select_data,
                         const_tag,aggregated_ownership,select_data::result_type>  filtered_data;
};
END_VIEW_DETAILS_NAMESPACE


template <class container,
          class flags,
          class const_tag,
          class iterator_tag,
          class ownership_tag_a,
          class ownership_tag_b
         >
class flagged_filter_view:
public VIEW_DETAILS_NAMESPACE::flagged_filter_namespace<container,flags,const_tag,
                                  iterator_tag,ownership_tag_a,ownership_tag_b>::filtered_data {
public:
  typedef VIEW_DETAILS_NAMESPACE::flagged_filter_namespace<container,flags,const_tag,
                                   iterator_tag,ownership_tag_a,ownership_tag_b> ns;

  typedef ctor_arg<container,const_tag,ownership_tag_a>::type ctor_arg_type;
  typedef ctor_arg<flags,    const_tag,ownership_tag_b>::type flag_arg_type;
  
  // default constructible
  explicit flagged_filter_view() {}

  flagged_filter_view(ctor_arg_type& cont, flag_arg_type& flag_cont)
    : ns::filtered_data( ns::filtered_pairs( ns::pair_view(cont,flag_cont),
                                             ns::select_flag() ),
                         ns::select_data() )
  {}
  
};

                                                                     
END_VIEW_NAMESPACE

#endif // _FLAGGED_FILTER_VIEW_H
