/* This file is part of the Views Template Library, written by
 * Gary Powell & Martin Weiser
 *
 * Copyright (c) 1999  Gary Powell
 * Copyright (c) 1999  Konrad Zuse Zentrum fuer Informationstechnik Berlin
 *
 * This material is provided "as is", with absolutely no warranty expressed
 * or implied. Any use is at your own risk.
 *
 * Permission to use or copy this software for any purpose is hereby granted 
 * without fee, provided the above notices are retained on all copies.
 * Permission to modify the code and to distribute modified code is granted,
 * provided the above notices are retained, and a notice that the code was
 * modified is included with the above copyright notice.
 */

#if !defined(_POLYMORPHIC_VIEW_HPP)
#define _POLYMORPHIC_VIEW_HPP 1

#if !defined(_POLYMORPHIC_VIEW_FWD_HPP)
#include "polymorphic_view_fwd.hpp"	// validate that the forward declaration matches the actual one.
#endif

#if !defined(_VIEW_TRAITS_HPP)
#include "details/view_traits.hpp"
#endif 

#if !defined(_TRANSFORM_VIEW_HPP)
#include "transform_view.hpp" 
#endif

#if !defined(_FUNCTORS_HPP)
#include "functors.hpp"
#endif

#if !defined(_STL_ALGORITHM_FWD_HPP)
#include "details/stl_algorithm_fwd.hpp"
#endif
  
BEGIN_VIEW_NAMESPACE
//
// A polymorphic view.
//

template <class container,
          class const_tag = const_view_tag,
          class ownership_tag = external_ownership>
class polymorphic_view
  : public transform_view<container,
                          dereference<const_traits<typename container::value_type>::type>,
                          const_tag,
                          ownership_tag,                                            
                          dereference<const_traits<typename container::value_type>::type>::result_type> {
public:
  typedef dereference<const_traits<typename container::value_type>::type> transform_type;
  typedef transform_view<container,transform_type,const_tag,ownership_tag,transform_type::result_type> inherited;

  // containers have to be default constructible
  explicit polymorphic_view() {}
  
  explicit polymorphic_view(inherited::ctor_arg_type& cont)
    : inherited(cont,transform_type()) {}
};


END_VIEW_NAMESPACE
#endif
// $Id: polymorphic_view.hpp,v 1.1 2000/11/20 14:10:50 bzfweise Exp $ 
