/* This file is part of the Views Template Library, written by
 * Gary Powell & Martin Weiser
 *
 * Copyright (c) 1999  Gary Powell
 * Copyright (c) 1999  Konrad Zuse Zentrum fuer Informationstechnik Berlin
 *
 * This material is provided "as is", with absolutely no warranty expressed
 * or implied. Any use is at your own risk.
 *
 * Permission to use or copy this software for any purpose is hereby granted 
 * without fee, provided the above notices are retained on all copies.
 * Permission to modify the code and to distribute modified code is granted,
 * provided the above notices are retained, and a notice that the code was
 * modified is included with the above copyright notice.
 */

#include "flagged_filter_view.hpp"

#include <cstdlib>
#include <vector>
#include <list>
#include <iostream>
#include <numeric>
#include <functional>

#include "dump.h"
#include "iterator_test.h"

using std::cout;
using std::cin;
using std::endl;
using std::vector;
using std::advance;

USING_VIEW_NAMESPACE

template <class it>
bool is_prime(unsigned p, it start, it end) {
  return (std::find_if(start,end,
                 std::compose1(std::bind1st(std::equal_to<unsigned>(),0),
                          std::bind1st(std::modulus<unsigned>(),p))       // p%x == 0
                 )==end);
}


int main()
{
  vector<unsigned> x;
  list<bool>  flag;

  typedef flagged_filter_view< vector<unsigned>,list<bool> > flagged_primes;
  flagged_primes primes(x,flag);
  
  
  for (unsigned i=2; i<30; ++i) {
    flag.push_back(is_prime(i,primes.begin(),primes.end()));
    x.push_back(i);
  }

  cout << "numbers " << endl;
  dump(cout, x);

  cout << "flagged Test: you should see only prime numbers:" << endl;
  dump(cout, primes);

  // iterator category test.
  flagged_primes::iterator iter_vv(primes.begin());
  advance(iter_vv, 2);

  cout << "reverse order test.:" << endl;
  rdump(cout, primes);

  forward_test(primes);
  const_forward_test(primes);
  reverse_test(primes);
  const_reverse_test(primes);
  random_access_test(primes);
  const_random_access_test(primes);


  cout << "type q <cr> to exit app. ";
  char c;
  do {
    cin >> c;
    cout << c;
  } while (c != 'q');
    
  cout << endl;
  return EXIT_SUCCESS;
}
